/*
 * log_dump: dump current system states
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <limits.h>
#include <time.h>
#include <unistd.h>
#include <dirent.h>
#include <libgen.h>
#include <system_info.h>
#include <tzplatform_config.h>
#include "shared/util.h"
#include "log_dump.h"
#include "dbus-handler.h"

#undef LOG_TAG
#define LOG_TAG          "LOG_DUMP"
#define LOG_DUMP_ROOT    tzplatform_getenv(TZ_SYS_CRASH_ROOT)
#define LOG_DUMP_DIR     tzplatform_getenv(TZ_SYS_ALLLOGS)
#define CRASH_DUMP_DIR   tzplatform_getenv(TZ_SYS_CRASH)
#define DUMP_SCRIPTS_DIR tzplatform_getenv(TZ_SYS_DUMPGEN)
#define SYSTEM_INFO_KEY_BUILD_STRING "http://tizen.org/system/build.string"

static const struct option opts[] = {
	{ "normal", no_argument, 0, OPT_NORMAL },
	{ "short",  no_argument, 0, OPT_SHORT  },
	{ "dbus",   no_argument, 0, OPT_DBUS   },
	{ 0, 0, 0, 0 }
};

static inline void usage(void)
{
	printf("Usage: log_dump [OPTION]\n");
	printf("Dump options:\n");
	printf("  %-10s %s (%s)\n", "--normal",
			"dump all logs", DUMP_SCRIPTS_DIR);
	printf("  %-10s %s\n", "--short",
			"dump systemstate only");
	printf("  %-10s %s\n", "--dbus",
			"activate dbus interface");
}

static int dump_scripts(void)
{
	struct dirent **dir_list = NULL;
	char command[PATH_MAX];
	int script_num, i;

	script_num = scandir(DUMP_SCRIPTS_DIR, &dir_list, NULL, NULL);
	if (script_num < 0) {
		LOGE("Failed to scandir");
		return -1;
	}

	for (i = 0; i < script_num; i++) {
		if (dir_list[i]->d_type != DT_REG)
			continue;

		snprintf(command, sizeof(command), "%s/%s %s",
				DUMP_SCRIPTS_DIR, dir_list[i]->d_name,
				LOG_DUMP_DIR);
		LOGD("%s", command);
		system_command(command);
	}

	for (i = 0; i < script_num; i++)
		free(dir_list[i]);
	free(dir_list);

	return 0;
}

int log_dump(int option)
{
	int ret;
	char *version_str = NULL;
	char *dump_dirname = NULL;
	char *crash_dirname = NULL;
	char timestr[80];
	char command[PATH_MAX];
	char dump_filename[NAME_MAX];
	time_t cur_time;
	struct tm loc_tm;

	/* Make debug directory */
	if (access(LOG_DUMP_DIR, F_OK) != 0) {
		ret = snprintf(command, sizeof(command),
				"/usr/bin/mkdir -p %s", LOG_DUMP_DIR);
		if (ret < 0) {
			LOGE("Failed to mkdir");
			return -1;
		}
		system_command(command);
	}

	/* Get timestamp */
	cur_time = time(NULL);
	localtime_r(&cur_time, &loc_tm);
	strftime(timestr, sizeof(timestr), "%Y%m%d%H%M%S", &loc_tm);

	/* Get version */
	ret = system_info_get_platform_string(SYSTEM_INFO_KEY_BUILD_STRING,
			&version_str);
	if (ret != SYSTEM_INFO_ERROR_NONE) {
		LOGE("Failed to system_info_get_platform_string");
		version_str = NULL;
	}

	/* Dump system states */
	ret = snprintf(command, sizeof(command),
			"/usr/bin/dump_systemstate -k -d -f "
			"%s/dump_systemstate_%s.log", LOG_DUMP_DIR, timestr);
	if (ret < 0) {
		LOGE("Failed to snprintf for command");
		goto exit;
	}
	system_command(command);

	/* Dump all logs */
	if (option == OPT_NORMAL)
		dump_scripts();

	if (version_str) {
		ret = snprintf(dump_filename, sizeof(dump_filename), "%s_%s",
				"log_dump", version_str);
		if (ret < 0) {
			LOGE("Failed to snprintf for dump path");
			goto exit;
		}
	} else {
		ret = snprintf(dump_filename, sizeof(dump_filename), "%s",
				"log_dump");
		if (ret < 0) {
			LOGE("Failed to snprintf for dump path");
			return -1;
		}
	}

	/* Compression */
	dump_dirname = strdup(LOG_DUMP_DIR);
	if (!dump_dirname) {
		LOGE("Failed to strdup for dump_dirname");
		goto exit;
	}

	if (option == OPT_NORMAL) {
		crash_dirname = strdup(CRASH_DUMP_DIR);
		if (!crash_dirname) {
			LOGE("Failed to strdup for dump_dirname");
			goto exit;
		}

		ret = snprintf(command, sizeof(command),
				"/bin/tar -zcf %s/%s%s.tar.gz -C %s %s %s",
				LOG_DUMP_ROOT, dump_filename, timestr,
				LOG_DUMP_ROOT, basename(dump_dirname),
				basename(crash_dirname));
		if (ret < 0) {
			LOGE("Failed to snprintf for command");
			goto exit;
		}
	} else {
		ret = snprintf(command, sizeof(command),
				"/bin/tar -zcf %s/%s%s.tar.gz -C %s %s",
				LOG_DUMP_ROOT, dump_filename, timestr,
				LOG_DUMP_ROOT, basename(dump_dirname));
		if (ret < 0) {
			LOGE("Failed to snprintf for command");
			goto exit;
		}
	}
	system_command(command);

	sync();

	/* Remove gatherd dump */
	ret = remove_dir(LOG_DUMP_DIR, 0);
	if (ret < 0) {
		LOGE("Failed to delete dump directory");
		goto exit;
	}
	if (option == OPT_NORMAL) {
		ret = remove_dir(CRASH_DUMP_DIR, 0);
		if (ret < 0) {
			LOGE("Failed to delete crash dump directory");
			goto exit;
		}
	}

	/* Further operations for log_dump here */

exit:
	if (version_str)
		free(version_str);
	if (dump_dirname)
		free(dump_dirname);
	if (crash_dirname)
		free(crash_dirname);

	return ret;
}

int main(int argc, char *argv[])
{
	int c, ret;
	int option;

	if (argc < 2) {
		usage();
		exit(EXIT_SUCCESS);
	}

	option = -1;
	while ((c = getopt_long_only(argc, argv, "", opts, NULL)) != -1) {
		switch (c) {
		case OPT_NORMAL:
			if (option >= 0) {
				usage();
				exit(EXIT_SUCCESS);
			}
			option = OPT_NORMAL;
			break;
		case OPT_SHORT:
			if (option >= 0) {
				usage();
				exit(EXIT_SUCCESS);
			}
			option = OPT_SHORT;
			break;
		case OPT_DBUS:
			if (option >= 0) {
				usage();
				exit(EXIT_SUCCESS);
			}
			option = OPT_DBUS;
			break;
		default:
			usage();
			exit(EXIT_SUCCESS);
			break;
		}
	}

	if (option == OPT_DBUS)
		ret = log_dump_dbus();
	else
		ret = log_dump(option);

	if (ret < 0)
		exit(EXIT_FAILURE);

	return 0;
}
